#include <Servo.h>          // include the Servo library
#include <NewPing.h>        // include the Ultrasonic sensor library



//-------------------------MOTOR-----------------------


// Motor controller pins
  
  // Motor A

#define ENA 5     //enable right motor
#define IN1 7     //right forward
#define IN2 8     //right reverse

  // Motor B

#define ENB 6     //enable left motor
#define IN3 9     //left reverse
#define IN4 11    //left forward

#define carSpeed 150        // possible speed range: 0-255 (but with our current power supply of one 9V battery you should not exceed 170)
#define carSpeed2 100       // optional, if you want to use two different speeds


//-------------------------COLOR SENSOR-----------------------


// Color sensor pins

#define S0 10
#define S1 4
#define S2 2
#define S3 1
#define sensorOut 0
#define LED 13

// Calibration values from the test run (5 cm distance to the sheet)

int redMin = 19;          //min value from white sheet
int redMax = 57;          //max value from black sheet
int greenMin = 24;
int greenMax = 71;
int blueMin = 20;
int blueMax = 65;

// Variables for Color Pulse Width Measurements

int redPW = 0;
int greenPW = 0;
int bluePW = 0;

// Variables for final color values

int redValue;
int greenValue;
int blueValue;


// Function to read Red Pulse Widths
int getRedPW() {

  // Set sensor to read Red only
  digitalWrite(S2,LOW);
  digitalWrite(S3,LOW);
  // Define integer to represent Pulse Width
  int PW;
  // Read the output Pulse Width
  PW = pulseIn(sensorOut, LOW);
  // Return the value
  return PW;
}

// Function to read Green Pulse Widths
int getGreenPW() {

  // Set sensor to read Green only
  digitalWrite(S2,HIGH);
  digitalWrite(S3,HIGH);
  // Define integer to represent Pulse Width
  int PW;
  // Read the output Pulse Width
  PW = pulseIn(sensorOut, LOW);
  // Return the value
  return PW;
}

// Function to read Blue Pulse Widths
int getBluePW() {

  // Set sensor to read Blue only
  digitalWrite(S2,LOW);
  digitalWrite(S3,HIGH);
  // Define integer to represent Pulse Width
  int PW;
  // Read the output Pulse Width
  PW = pulseIn(sensorOut, LOW);
  // Return the value
  return PW;
}



//-------------------------IR (object detection short range)------------------------------


// IR Proximity pins

#define LS A4               //left sensor
#define RS A5               //right sensor


//-------------------------ULTRASONIC (object detection long range)-----------------------



// Ultrasonic pins and variables

#define ECHO_PIN A0                           // sensor receive pin (pulse returns)
#define TRIGGER_PIN A1                        // sensor send pin (pulse sent)
#define MAX_DISTANCE 100

NewPing sonar(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE);     // Variables for the Ultrasonic sensor 



//-------------------------SERVO (panning the Ultrasonic and IRs)-------------------------



Servo myservo;                                          // Create servo object to control the servo
int pos = 0;



//-------------------------CAR MOVEMENTS--------------------------------------------------


//Default movements

void forward()
{
  analogWrite(ENA, carSpeed);
  analogWrite(ENB, carSpeed);
  digitalWrite(IN1, HIGH);          // right forward
  digitalWrite(IN2, LOW);           // right back
  digitalWrite(IN3, LOW);           // left back
  digitalWrite(IN4, HIGH);          // left forward
  Serial.println("Forward");
}

void back() 
{
  analogWrite(ENA, carSpeed);
  analogWrite(ENB, carSpeed);
  digitalWrite(IN1, LOW);          
  digitalWrite(IN2, HIGH);
  digitalWrite(IN3, HIGH);
  digitalWrite(IN4, LOW);
  Serial.println("Back");
}

void right() 
{
  analogWrite(ENA, carSpeed2);
  analogWrite(ENB, carSpeed2);
  digitalWrite(IN1, LOW);
  digitalWrite(IN2, HIGH);
  digitalWrite(IN3, LOW);
  digitalWrite(IN4, HIGH); 
  Serial.println("Right");
}

void left() 
{
  analogWrite(ENA, carSpeed2);
  analogWrite(ENB, carSpeed2);
  digitalWrite(IN1, HIGH);
  digitalWrite(IN2, LOW);
  digitalWrite(IN3, HIGH);
  digitalWrite(IN4, LOW);
  Serial.println("Left");
}

void stop() 
{
  digitalWrite(ENA, LOW);
  digitalWrite(ENB, LOW);
  Serial.println("Stop!");
}


void setup() 
{

//Servo settings to start initial object search

  Serial.begin(9600);    
  myservo.attach(3);                          // attach servo on pin 3 to servo object (PWM)
  {
    for(pos = 90; pos <= 180; pos += 1){
      myservo.write(pos);
      delay(15);
    }for(pos = 180; pos >= 0; pos -= 1){
      myservo.write(pos);
      delay(15);
    }for(pos = 0; pos <= 90; pos += 1){
      myservo.write(pos);
      delay(15);
    }
  } 

  pinMode(LS, INPUT);               //IR proximity
  pinMode(RS, INPUT);               //IR proximity
      
  
  pinMode(IN1, OUTPUT);             //Wheels..
  pinMode(IN2, OUTPUT);
  pinMode(IN3, OUTPUT);
  pinMode(IN4, OUTPUT);
  pinMode(ENA, OUTPUT);
  pinMode(ENB, OUTPUT);


  pinMode(S0, OUTPUT);              //Color sensor...
  pinMode(S1, OUTPUT);
  pinMode(S2, OUTPUT);
  pinMode(S3, OUTPUT);
  pinMode(LED, OUTPUT);
  pinMode(sensorOut, INPUT);
  digitalWrite(S0,HIGH);
  digitalWrite(S1,LOW);
  analogWrite(LED, HIGH);

  stop();
}

void loop(){ 

//--------------------ENABLING THE COLOR SENSOR-----------------

{
  // Read Red value
  redPW = getRedPW();
  redValue = map(redPW, redMin, redMax, 255, 0);
  delay(200);

  // Read Green value
  greenPW = getGreenPW();
  greenValue = map(greenPW, greenMin, greenMax, 255, 0);
  delay(200);

  // Read Blue value
  bluePW = getBluePW();
  blueValue = map(bluePW, blueMin, blueMax, 255, 0);
  delay(200);
}


//--------------------OBJECT FOLLOWING LOGIC-----------------

  
  delay(50);
  unsigned int distance = sonar.ping_cm();      //Enabling Ultrasonic sensor
  Serial.print("distance");
  Serial.println(distance);
  
  
  int Right_Value = digitalRead(RS);            //Enabling IR sensors
  int Left_Value = digitalRead(LS);


  Serial.print("RIGHT");
  Serial.println(Right_Value);
  Serial.print("LEFT");
  Serial.println(Left_Value); 


  //Movement commands for Ultrasonic and IR only - adjust the numbers as it suits your project

  if((Right_Value == 1) && (distance >= 5 && distance <= 30) && (Left_Value == 1)){    
    forward();
  }else if((Right_Value == 0) && (Left_Value == 1)){
    left();
    delay(100);
  }else if((Right_Value == 1) && (Left_Value == 0)){
    right();
    delay(100);
  }else if(distance > 1 && distance < 5){
    stop();


  //Movement commands including the color sensor

/*
  if((redValue > 200) ){         //Core statement
    back();                      //Add defaut movements and delays as you please, try also and combine this if-statement with the ones above for a complete sequence
    delay(2000);
*/
  

}
}
